# frozen_string_literal: true

User.class_eval do
  field :main_applicant_id, type: BSON::ObjectId
  field :reservation_holder_id, type: BSON::ObjectId
  field :lessor_id, type: BSON::ObjectId
  field :gender, type: String

  after_save :hermes_after_actions

  def main_applicant
    Hermes::Applicant.find_by(id: main_applicant_id)
  end

  def reservation_holder
    Hermes::ReservationHolder.find(reservation_holder_id)
  end

  def lessor
    Hermes::Lessor.find(lessor_id)
  rescue
    nil
  end

  def lessor=(les)
    self.lessor_id = les.id
  end

  def has_lessor?
    lessor_id.present?
  end

  def update_rights
    deleted_group_ids = (group_ids_was || []) - (group_ids || []) 
    added_group_ids = (group_ids || []) - (group_ids_was || []) 

    deleted_group_ids.each do |id|
      if data = create_right_data(id)
        ur = Hermes::UserRight.find_by(user_id: id, group_id: id)
        ur.delete if ur
      end
    end
    
    added_group_ids.each do |id|
      if data = create_right_data(id)
        ur = Hermes::UserRight.find_by(user_id: id, group_id: id)
        Hermes::UserRight.new(data) unless ur
      end
    end
  end

  def create_right_data(id)
    return unless group = Group.find(id)

    right = case group.name
      when /^entity_manager_/
        'administrator'
      when /^entity_supervisor_/
        'user'
      when /^entity_/
        'reader'
      else
        nil
      end

      if right 
        matching_type = {
          administrator: 'manager_group_id',
          user: 'supervisor_group_id',
          reader: 'group_id',
        }

        query = {}
        query[matching_type[right.to_sym]] = group.id

        entity = Hermes::Entity.find_by(query)
        lessor = Hermes::Lessor.find_by(query) if !entity

        o = entity || lessor

        return {
          user_id: id,
          entity_id: entity ? o.id : nil,
          lessor_id: entity ? nil : o.id,
          group_id: group.id,
          title: o.title,
          right: right,
          type: entity ? 'entity' : 'lessor'
        }
      end
    nil
  end
  
  def hermes_after_actions
    update_main_applicant_name if name_changed?
    update_main_applicant_email if email_changed?
    main_applicant.save if main_applicant_will_move?
    update_rights if group_ids_changed?
  end

  def name_changed?
    first_name_changed? || last_name_changed?
  end

  def update_main_applicant_name
    main_applicant&.name_will_change = true
  end

  def update_main_applicant_email
    main_applicant&.email_will_change = true
  end

  def main_applicant_will_move?
    main_applicant&.name_will_change || main_applicant&.email_will_change
  end
end
